package lieAlgebra;

import java.io.IOException;
import java.io.PrintWriter;
import java.text.ParseException;
import java.util.Vector;

import pal.math.MersenneTwisterFast;
import pal.substmodel.GeneralREVRateMatrix;
import pal.substmodel.LieMarkovModel;
import pal.substmodel.NeoRateMatrix;
import pal.substmodel.OrthogonalizedLieMarkovRateMatrix;

//import mdwUtils.TeePrintStream;

/**
 * Holds the options set by command line args and options.
 * 
 * @author Michael
 * 
 * Licensed under the Creative Commons Zero (CC0) license (https://creativecommons.org/publicdomain/zero/1.0/)
 *
 */

public class Configuration {
	
	public void usage() {
		System.out.println("Command line options:");
		System.out.println("-m#: Which 'main' to run. Default 0. 1= RateMatrixHandler test, 2=print rays, 3=multiple runs to test always get same result.");
		System.out.println("-n#: Number of trials to run (for -m3 only)");
		System.out.println("-s#: RNG seed (for -m3 only)");
		System.out.println("-o<filename>: output to file and stdout");
		System.out.println("-M<number list>: which models to run");
		System.out.println("-F<number list>: which data files to run");
		System.out.println("-v: verbose output");
		System.out.println("-i: use invariant sites model");
		System.out.println("-g: use gamma rates model");
		System.out.println("-t: print elapsed time");
		System.out.println("-r: print likelihoods relative to first model evaluated");
		System.out.println("-f<filename>: input file name (can be used multiple times, makes -F redundant.)");
		System.out.println("Optimisation method options: (used 'mixed' by default: Approximate followed by Conjugate");
		System.out.println("-O: use OrthogonalSearch to optimise.");
		System.out.println("-C: use ConjugateDirectionSearch only to optimise.");
		System.out.println("-B: use ConjugateDirectionSearch, falling back on Orthogonal if Conjugate fails");
		System.out.println("-A: use approximate OrthogonalSearch (same method, lower accuracy stopping threshold.)");
		System.out.println("-P: use paranoid search: ConjugateDirection, then OrthogonalSearch, then ConjugateDirection");
		System.out.println("    all at full accuracy.");
		System.out.println();
		System.out.println("By default, start with approximate orthogornal (-A) and refine with ConjugateDirection (-C).");
		System.out.println();
		System.out.println("Model numbers:");
		System.out.println("Parameterization: x=cubic, x+100=Fourier-Motzkin, x+200=octahedral, x+300=spherical, x+400=ext spherical.");
		System.out.println(" 0   3.3a");
		System.out.println(" 1   4.4a");
		System.out.println(" 2   6.7a");
		System.out.println(" 3   9.2b");
		System.out.println(" 4  12.12");
		System.out.println(" 5 RY2.2b     36 WS2.2b     67 MK2.2b ");
		System.out.println(" 6 RY3.3b     37 WS3.3b     68 MK3.3b ");
		System.out.println(" 7 RY3.3c     38 WS3.3c     69 MK3.3c ");
		System.out.println(" 8 RY3.4      39 WS3.4      70 MK3.4  ");
		System.out.println(" 9 RY4.4b     40 WS4.4b     71 MK4.4b ");
		System.out.println("10 RY4.5a     41 WS4.5a     72 MK4.5a ");
		System.out.println("11 RY4.5b     42 WS4.5b     73 MK4.5b ");
		System.out.println("12 RY5.6a     43 WS5.6a     74 MK5.6a ");
		System.out.println("13 RY5.6b     44 WS5.6b     75 MK5.6b ");
		System.out.println("14 RY5.7a     45 WS5.7a     76 MK5.7a ");
		System.out.println("15 RY5.7b     46 WS5.7b     77 MK5.7b ");
		System.out.println("16 RY5.7c     47 WS5.7c     78 MK5.7c ");
		System.out.println("17 RY5.11a    48 WS5.11a    79 MK5.11a");
		System.out.println("18 RY5.11b    49 WS5.11b    80 MK5.11b");
		System.out.println("19 RY5.11c    50 WS5.11c    81 MK5.11c");
        System.out.println("20 RY5.16     51 WS5.16     82 MK5.16 ");
        System.out.println("21 RY6.6      52 WS6.6      83 MK6.6  ");
        System.out.println("22 RY6.7b     53 WS6.7b     84 MK6.7b ");
        System.out.println("23 RY6.8a     54 WS6.8a     85 MK6.8a ");
        System.out.println("24 RY6.8b     55 WS6.8b     86 MK6.8b ");
        System.out.println("25 RY6.17a    56 WS6.17a    87 MK6.17a");
        System.out.println("26 RY6.17b    57 WS6.17b    88 MK6.17b");
        System.out.println("27 RY8.8      58 WS8.8      89 MK8.8  ");
        System.out.println("28 RY8.10a    59 WS8.10a    90 MK8.10a");
        System.out.println("29 RY8.10b    60 WS8.10b    91 MK8.10b");
        System.out.println("30 RY8.16     61 WS8.16     92 MK8.16 ");
        System.out.println("31 RY8.17     62 WS8.17     93 MK8.17 ");
        System.out.println("32 RY8.18     63 WS8.18     94 MK8.18 ");
        System.out.println("33 RY9.20a    64 WS9.20a    95 MK9.20a");
        System.out.println("34 RY10.12    65 WS10.12    96 MK10.12");
        System.out.println("35 RY10.34    66 WS10.34    97 MK10.34");
		System.out.println("500 JC=1.1     507 F81");
		System.out.println("501 K80=2.2b   508 HKY");
		System.out.println("502 TrNef=3.3c 509 TrN");
		System.out.println("503 K81=3.3a   510 K81uf");
		System.out.println("504 TVMef      511 TVM");
		System.out.println("505 TIMef      512 TIM");
		System.out.println("506 SYM        513 GTR");
	}
	
	protected String[] masterFileList = new String[]{"53humans.phy", "acorus15.phy", "cormorants.nex", "rokas.nex", "z.11x2178.nex", "buttercup.nex", "M4310.nex"};
	public final String BASEDIR = "./sequences/";
	public static final float PARAM_UPPER_BOUND = 500;
	
	public TestModel[] modelList = null;
	public String[] fileList = null;
	public int nTrials=10;
	public long seed = 4; // http://xkcd.com/221/
	public boolean invariableSites = false; // -i option
	public boolean gammaRateDistribution = false; // -g option
	public boolean printToFile;
	public boolean verbose;
	public boolean altMain; // if true, we're not doing the standard run
	public boolean printRelative=false; // -r option
	public int main = 0; // which 'main' program to run. Can do arbitrary stuff for main!=0.
	public PrintWriter output = null;
	public boolean printTime = false;
	protected static Vector<TestModel> masterModelList = new Vector<TestModel>();
	protected int LAST_DEFAULT_MODEL = 100; // if now list of models to run, run 0 up to this number
	public enum SearchStrategy {MIXED, CONJUGATE, ORTHOGONAL, PARANOID, FALLBACK, APPROXIMATE};
	public SearchStrategy search=SearchStrategy.MIXED; // -C, -O, -P, -B, -A options
	/*
	 * Search strategies:
	 * MIXED = Approximate search followed by Conjugate search from that point
	 * CONJUGATE = Conjugate direction search
	 * ORTHOGONAL = orthogonal search
	 * PARANOID = conjugate search, then orthogonal search from its solution, then conjugate search again.
	 *            Very slow but thorough
	 * FALLBACK = conjugate search, but falls back to (orthogonal then conjugate) if first search fails.
	 * APPROXIMATE = orthogonal but only to two digit accuracy. Public visibility of this is really
	 *            just for timing tests.   
	 * TODO: Contemplate treating search strategies like models and data sets: list of numbers.         
	 */
	
	static {
		populateModelList();
	}
	

	// Primarily to give SimpleConfigutation constructor something to call.
	protected Configuration() {
		MersenneTwisterFast.forbidUnseeded();
	}
	
	public Configuration(String[] args) {
		this();
		boolean bad = false;
		String outputFileName=null;
		int[] runFiles = null;
		int[] runModels = null;
		
		String stringArg = null;
		boolean filesFromCommandLine = false;
		for (String arg : args) {
			if (arg.charAt(0)!='-') bad = true;
			char flag = arg.charAt(1);
			int count=-1;
//			float real = Float.NaN;
			if (arg.length()>2) {
//				real = -1;
				count = -1;
				stringArg = arg.substring(2);
				try {
//					real = Float.valueOf(arg.substring(2));
					count = Integer.valueOf(arg.substring(2));
				} catch (NumberFormatException e) {
					// ignore - must have been a float or string
				}
				
			}
			
			switch (flag) {
			case 'v' :  verbose = true;
		    			break;
			case 'n' :  nTrials = count;
						break;
			case 's' :  seed = count;
						break;
			case 'o' :  printToFile = true;
						outputFileName = stringArg;
						break;
			case 'i' :  invariableSites = true;
						break;
			case 'g' :  gammaRateDistribution = true;
						break;
			case 'r' :  printRelative = true;
						break;
			case 'O' :  search=SearchStrategy.ORTHOGONAL;
						break;
			case 'C' :  search=SearchStrategy.CONJUGATE;
						break;
			case 'P' :  search=SearchStrategy.PARANOID;
						break;
			case 'A' :  search=SearchStrategy.APPROXIMATE;
						break;
			case 'B' :  search=SearchStrategy.FALLBACK;
			break;

			case 'M' : 	try {
							runModels = pal.misc.Utils.parseNumberList(stringArg);
						} catch (ParseException e) {
							e.printStackTrace();
							bad = true;
						}
						break;
			case 'F' : 	try {
							runFiles = pal.misc.Utils.parseNumberList(stringArg);
						} catch (ParseException e) {
							e.printStackTrace();
							bad = true;
						}
						break;
			case 'f' :  filesFromCommandLine = true;
						if (fileList==null) {
							fileList = new String[]{stringArg};
						} else {
							String[] newList = new String[fileList.length+1];
							for (int i=0; i<fileList.length; i++) newList[i] = fileList[i];
							newList[fileList.length] = stringArg;
						}
						break;
			case 'm' :  main = count;
						altMain = (main != 0);
						break;	
			case 't' :  printTime = true;
						break;
			default: 	System.out.println("Unrecognized flag: "+flag);
						bad = true;
			}
		}
		if (bad && !altMain) {
			usage();
			System.exit(1);
		}
		//try {
			if (printToFile) {
				/*
				 * Removed to allow Creative Commons Zero licensing. TeePrintStream is available from
				 * http://www.darwinsys.com/
				 */
				// output = new PrintWriter(new TeePrintStream(System.out, outputFileName,true));
				output = new PrintWriter(System.out, true);
			} else {
				output = new PrintWriter(System.out, true);
			}
		//} catch (IOException e) {
		//	e.printStackTrace();
		//	System.exit(1);
		//}
		if (runModels == null) {
			for (int i=0; i<LAST_DEFAULT_MODEL; i++) modelList[i] = masterModelList.get(i);
		} else {
			modelList = new TestModel[runModels.length];
			for (int i=0; i<runModels.length; i++) modelList[i] = masterModelList.get(runModels[i]);
		}
		if (!filesFromCommandLine) {
		if (runFiles == null) {
			fileList = masterFileList;
		} else {
			fileList = new String[runFiles.length];
			for (int i=0; i<runFiles.length; i++) fileList[i] = masterFileList[runFiles[i]];
		}
		}
		output.print("Command line: ");
		for (String arg : args) output.printf("%s ", arg);
		output.println();
	}
	
	static void populateModelList() {
		/*
		 * Models 0-97: LM models, cubic parameterization (which I expect to be the best)
		 * Models 100-197: LM models, Fourier-Motzkin parameterization
		 * Models 200-297: LM models, octahedral parameterization
		 * Models 300-397: LM models, spherical parameterization
		 * Models 400-497: LM models, extended spherical parameterization
		 * Models 500-513: ModelTest time reversible model heirarchy
		 * TODO: add ray parameterization (which would bump Modeltest to the 600s)
		 */
		
		// No RY, WS, MK variants on these ones:
		final String[] SYMMETRIC_MODELS = new String[] {"3.3a","4.4a","6.7a","9.20b","12.12"};
		// RY, WS, MK variants for these ones:
		final String[] ASYMMETRIC_MODELS = new String[] {"2.2b","3.3b","3.3c","3.4","4.4b","4.5a","4.5b",
				"5.6a","5.6b","5.7a","5.7b","5.7c","5.11a","5.11b","5.11c","5.16","6.6","6.7b","6.8a",
				"6.8b","6.17a","6.17b","8.8","8.10a","8.10b","8.16","8.17","8.18","9.20a","10.12","10.34"};
		final String[] PAIRINGS = new String[]{"RY","WS","MK"};
		// 
		// Cubic parameterization, models 0-97 (plus two dummies)
		for (String model : SYMMETRIC_MODELS) {
			masterModelList.add(new TestModel("c".concat(model), new CubicLieMarkovRateMatrix(model,"RY"),false,true));
		}
		for (String pairing : PAIRINGS) {
			for (String model : ASYMMETRIC_MODELS) {
				masterModelList.add(
						new TestModel(
								"c".concat(pairing).concat(model), 
								new CubicLieMarkovRateMatrix(model,pairing),
								false,
								true
						)
				);
			}	
		}	
		// Dummies to keep corresponding models 100 apart:
		masterModelList.add(new TestModel("dummy", GeneralREVRateMatrix.createJC(), false,false));
		masterModelList.add(new TestModel("dummy", GeneralREVRateMatrix.createJC(), false,false));
		// Fourier-Motzkin parameterization, models 100-197 (plus two dummies)
		for (String model : SYMMETRIC_MODELS) {
			masterModelList.add(new TestModel("f".concat(model), getFMmodel(model),false,true));
		}
		for (String pairing : PAIRINGS) {
			for (String model : ASYMMETRIC_MODELS) {
				String fullModel = pairing.concat(model);
				masterModelList.add(new TestModel("f".concat(fullModel), getFMmodel(fullModel),false,true));
			}
		}
		// Dummies to keep corresponding models 100 apart:
		masterModelList.add(new TestModel("dummy", GeneralREVRateMatrix.createJC(), false,false));
		masterModelList.add(new TestModel("dummy", GeneralREVRateMatrix.createJC(), false,false));
		// Octahedral polar parameterization, models 200-297 (plus two dummies)
		for (String model : SYMMETRIC_MODELS) {
			masterModelList.add(new TestModel("o".concat(model), new OctahedralLieMarkovRateMatrix(model,"RY"),false,true));
		}
		for (String pairing : PAIRINGS) {
			for (String model : ASYMMETRIC_MODELS) {
				masterModelList.add(
						new TestModel(
								"o".concat(pairing).concat(model), 
								new OctahedralLieMarkovRateMatrix(model,pairing),
								false,
								true
						)
				);
			}	
		}
		// Dummies to keep corresponding models 100 apart:
		masterModelList.add(new TestModel("dummy", GeneralREVRateMatrix.createJC(), false,false));
		masterModelList.add(new TestModel("dummy", GeneralREVRateMatrix.createJC(), false,false));
		// Spherical polar parameterization, models 300-397 (plus two dummies)
		for (String model : SYMMETRIC_MODELS) {
			masterModelList.add(new TestModel("s".concat(model), new SphericalLieMarkovRateMatrix(model,"RY"),false,true));
		}
		for (String pairing : PAIRINGS) {
			for (String model : ASYMMETRIC_MODELS) {
				masterModelList.add(
						new TestModel(
								"s".concat(pairing).concat(model), 
								new SphericalLieMarkovRateMatrix(model,pairing),
								false,
								true
						)
				);
			}	
		}	
		// Dummies to keep corresponding models 100 apart:
		masterModelList.add(new TestModel("dummy", GeneralREVRateMatrix.createJC(), false,false));
		masterModelList.add(new TestModel("dummy", GeneralREVRateMatrix.createJC(), false,false));
		// Extended Spherical polar parameterization, models 400-497 (plus two dummies)
		for (String model : SYMMETRIC_MODELS) {
			masterModelList.add(new TestModel("e".concat(model), new ExtendedSphericalLieMarkovRateMatrix(model,"RY"),false,true));
		}
		for (String pairing : PAIRINGS) {
			for (String model : ASYMMETRIC_MODELS) {
				masterModelList.add(
						new TestModel(
								"e".concat(pairing).concat(model), 
								new ExtendedSphericalLieMarkovRateMatrix(model,pairing),
								false,
								true
						)
				);
			}	
		}	
		// Dummies to keep corresponding models 100 apart:
		masterModelList.add(new TestModel("dummy", GeneralREVRateMatrix.createJC(), false,false));
		masterModelList.add(new TestModel("dummy", GeneralREVRateMatrix.createJC(), false,false));

		
		// Comparison Modeltest (time reversible) models. This is the full 14 models of the ModelTest heirarchy.
		// Some overlap with LM models (K80, K81, and JC had I implemented it as a LM model) but
		// they are implemented differently, so act as a sanity check.
		// Models 500-513
		masterModelList.add(new TestModel("JC",   GeneralREVRateMatrix.createJC(),      false,false));   // 500
		masterModelList.add(new TestModel("K80",  ModifiedGREVRateMatrix.modCreateHKY(),false,false));   // 501
		masterModelList.add(new TestModel("TrNef",ModifiedGREVRateMatrix.modCreateTrN(),false,false));   // 502 
		masterModelList.add(new TestModel("K81",  ModifiedGREVRateMatrix.modCreateK81(),false,false));   // 503
		masterModelList.add(new TestModel("TVMef",ModifiedGREVRateMatrix.modCreateTVM(),false,false));   // 504
		masterModelList.add(new TestModel("TIMef",ModifiedGREVRateMatrix.modCreateTIM(),false,false));   // 505
		masterModelList.add(new TestModel("SYM",  ModifiedGREVRateMatrix.modCreateGTR(),false,false));   // 506

		masterModelList.add(new TestModel("F81",  GeneralREVRateMatrix.createJC(),      true, false));   // 507
		masterModelList.add(new TestModel("HKY",  ModifiedGREVRateMatrix.modCreateHKY(),true, false));   // 508
		masterModelList.add(new TestModel("TrN",  ModifiedGREVRateMatrix.modCreateTrN(),true, false));   // 509
		masterModelList.add(new TestModel("K81uf",ModifiedGREVRateMatrix.modCreateK81(),true, false));   // 510
		masterModelList.add(new TestModel("TVM",  ModifiedGREVRateMatrix.modCreateTVM(),true, false));   // 511
		masterModelList.add(new TestModel("TIM",  ModifiedGREVRateMatrix.modCreateTIM(),true, false));   // 512
		masterModelList.add(new TestModel("GTR",  ModifiedGREVRateMatrix.modCreateGTR(),true, false));   // 513
	}
	
	// Quick hack until Fourier-Motzkin models are integrated into new framework
	static NeoRateMatrix getFMmodel(String model) {
		switch(model) {
		case "RY2.2b" : return OrthogonalizedLieMarkovRateMatrix.getM2r2bRY();
		case "3.3a"   : return OrthogonalizedLieMarkovRateMatrix.getM3r3a();
		case "RY3.3b" : return OrthogonalizedLieMarkovRateMatrix.getM3r3bRY();
		case "RY3.3c" : return OrthogonalizedLieMarkovRateMatrix.getM3r3cRY();
		case "RY3.4"  : return OrthogonalizedLieMarkovRateMatrix.getM3r4RY();
		case "4.4a"   : return OrthogonalizedLieMarkovRateMatrix.getM4r4a();
		case "RY4.4b" : return OrthogonalizedLieMarkovRateMatrix.getM4r4bRY();
		case "RY4.5a" : return OrthogonalizedLieMarkovRateMatrix.getM4r5aRY();
		case "RY4.5b" : return OrthogonalizedLieMarkovRateMatrix.getM4r5bRY();
		case "RY5.6a" : return OrthogonalizedLieMarkovRateMatrix.getM5r6aRY();
		case "RY5.6b" : return OrthogonalizedLieMarkovRateMatrix.getM5r6bRY();
		case "RY5.7a" : return OrthogonalizedLieMarkovRateMatrix.getM5r7aRY();
		case "RY5.7b" : return OrthogonalizedLieMarkovRateMatrix.getM5r7bRY();
		case "RY5.7c" : return OrthogonalizedLieMarkovRateMatrix.getM5r7cRY();
		case "RY5.11a": return OrthogonalizedLieMarkovRateMatrix.getM5r11aRY();
		case "RY5.11b": return OrthogonalizedLieMarkovRateMatrix.getM5r11bRY();
		case "RY5.11c": return OrthogonalizedLieMarkovRateMatrix.getM5r11cRY();
		case "RY5.16" : return OrthogonalizedLieMarkovRateMatrix.getM5r16RY();
		case "RY6.6"  : return OrthogonalizedLieMarkovRateMatrix.getM6r6RY();
		case "6.7a"   : return OrthogonalizedLieMarkovRateMatrix.getM6r7a();
		case "RY6.7b" : return OrthogonalizedLieMarkovRateMatrix.getM6r7bRY();
		case "RY6.8a" : return OrthogonalizedLieMarkovRateMatrix.getM6r8aRY();
		case "RY6.8b" : return OrthogonalizedLieMarkovRateMatrix.getM6r8bRY();
		case "RY6.17a": return OrthogonalizedLieMarkovRateMatrix.getM6r17aRY();
		case "RY6.17b": return OrthogonalizedLieMarkovRateMatrix.getM6r17bRY();
		case "RY8.8"  : return OrthogonalizedLieMarkovRateMatrix.getM8r8RY();
		case "RY8.10a": return OrthogonalizedLieMarkovRateMatrix.getM8r10aRY();
		case "RY8.10b": return OrthogonalizedLieMarkovRateMatrix.getM8r10bRY();
		case "RY8.16" : return OrthogonalizedLieMarkovRateMatrix.getM8r16RY();
		case "RY8.17" : return OrthogonalizedLieMarkovRateMatrix.getM8r17RY();
		case "RY8.18" : return OrthogonalizedLieMarkovRateMatrix.getM8r18RY();
		case "RY9.20a": return OrthogonalizedLieMarkovRateMatrix.getM9r20aRY();
		case "9.20b"  : return OrthogonalizedLieMarkovRateMatrix.getM9r20bRY();
		case "RY10.12": return OrthogonalizedLieMarkovRateMatrix.getM10r12RY();
		case "RY10.34": return OrthogonalizedLieMarkovRateMatrix.getM10r34RY();
		case "12.12"  : return LieMarkovModel.getModel12_12();             // 12.12 not implemented by Fourier-Motzkin so hack a replacement.
		case "WS2.2b" : return OrthogonalizedLieMarkovRateMatrix.getM2r2bWS();
		case "WS3.3b" : return OrthogonalizedLieMarkovRateMatrix.getM3r3bWS();
		case "WS3.3c" : return OrthogonalizedLieMarkovRateMatrix.getM3r3cWS();
		case "WS3.4"  : return OrthogonalizedLieMarkovRateMatrix.getM3r4WS();
		case "WS4.4b" : return OrthogonalizedLieMarkovRateMatrix.getM4r4bWS();
		case "WS4.5a" : return OrthogonalizedLieMarkovRateMatrix.getM4r5aWS();
		case "WS4.5b" : return OrthogonalizedLieMarkovRateMatrix.getM4r5bWS();
		case "WS5.6a" : return OrthogonalizedLieMarkovRateMatrix.getM5r6aWS();
		case "WS5.6b" : return OrthogonalizedLieMarkovRateMatrix.getM5r6bWS();
		case "WS5.7a" : return OrthogonalizedLieMarkovRateMatrix.getM5r7aWS();
		case "WS5.7b" : return OrthogonalizedLieMarkovRateMatrix.getM5r7bWS();
		case "WS5.7c" : return OrthogonalizedLieMarkovRateMatrix.getM5r7cWS();
		case "WS5.11a": return OrthogonalizedLieMarkovRateMatrix.getM5r11aWS();
		case "WS5.11b": return OrthogonalizedLieMarkovRateMatrix.getM5r11bWS();
		case "WS5.11c": return OrthogonalizedLieMarkovRateMatrix.getM5r11cWS();
		case "WS5.16" : return OrthogonalizedLieMarkovRateMatrix.getM5r16WS();
		case "WS6.6"  : return OrthogonalizedLieMarkovRateMatrix.getM6r6WS();
		case "WS6.7b" : return OrthogonalizedLieMarkovRateMatrix.getM6r7bWS();
		case "WS6.8a" : return OrthogonalizedLieMarkovRateMatrix.getM6r8aWS();
		case "WS6.8b" : return OrthogonalizedLieMarkovRateMatrix.getM6r8bWS();
		case "WS6.17a": return OrthogonalizedLieMarkovRateMatrix.getM6r17aWS();
		case "WS6.17b": return OrthogonalizedLieMarkovRateMatrix.getM6r17bWS();
		case "WS8.8"  : return OrthogonalizedLieMarkovRateMatrix.getM8r8WS();
		case "WS8.10a": return OrthogonalizedLieMarkovRateMatrix.getM8r10aWS();
		case "WS8.10b": return OrthogonalizedLieMarkovRateMatrix.getM8r10bWS();
		case "WS8.16" : return OrthogonalizedLieMarkovRateMatrix.getM8r16WS();
		case "WS8.17" : return OrthogonalizedLieMarkovRateMatrix.getM8r17WS();
		case "WS8.18" : return OrthogonalizedLieMarkovRateMatrix.getM8r18WS();
		case "WS9.20a": return OrthogonalizedLieMarkovRateMatrix.getM9r20aWS();
		case "WS10.12": return OrthogonalizedLieMarkovRateMatrix.getM10r12WS();
		case "WS10.34": return OrthogonalizedLieMarkovRateMatrix.getM10r34WS();
		case "MK2.2b" : return OrthogonalizedLieMarkovRateMatrix.getM2r2bMK();
		case "MK3.3b" : return OrthogonalizedLieMarkovRateMatrix.getM3r3bMK();
		case "MK3.3c" : return OrthogonalizedLieMarkovRateMatrix.getM3r3cMK();
		case "MK3.4"  : return OrthogonalizedLieMarkovRateMatrix.getM3r4MK();
		case "MK4.4b" : return OrthogonalizedLieMarkovRateMatrix.getM4r4bMK();
		case "MK4.5a" : return OrthogonalizedLieMarkovRateMatrix.getM4r5aMK();
		case "MK4.5b" : return OrthogonalizedLieMarkovRateMatrix.getM4r5bMK();
		case "MK5.6a" : return OrthogonalizedLieMarkovRateMatrix.getM5r6aMK();
		case "MK5.6b" : return OrthogonalizedLieMarkovRateMatrix.getM5r6bMK();
		case "MK5.7a" : return OrthogonalizedLieMarkovRateMatrix.getM5r7aMK();
		case "MK5.7b" : return OrthogonalizedLieMarkovRateMatrix.getM5r7bMK();
		case "MK5.7c" : return OrthogonalizedLieMarkovRateMatrix.getM5r7cMK();
		case "MK5.11a": return OrthogonalizedLieMarkovRateMatrix.getM5r11aMK();
		case "MK5.11b": return OrthogonalizedLieMarkovRateMatrix.getM5r11bMK();
		case "MK5.11c": return OrthogonalizedLieMarkovRateMatrix.getM5r11cMK();
		case "MK5.16" : return OrthogonalizedLieMarkovRateMatrix.getM5r16MK();
		case "MK6.6"  : return OrthogonalizedLieMarkovRateMatrix.getM6r6MK();
		case "MK6.7b" : return OrthogonalizedLieMarkovRateMatrix.getM6r7bMK();
		case "MK6.8a" : return OrthogonalizedLieMarkovRateMatrix.getM6r8aMK();
		case "MK6.8b" : return OrthogonalizedLieMarkovRateMatrix.getM6r8bMK();
		case "MK6.17a": return OrthogonalizedLieMarkovRateMatrix.getM6r17aMK();
		case "MK6.17b": return OrthogonalizedLieMarkovRateMatrix.getM6r17bMK();
		case "MK8.8"  : return OrthogonalizedLieMarkovRateMatrix.getM8r8MK();
		case "MK8.10a": return OrthogonalizedLieMarkovRateMatrix.getM8r10aMK();
		case "MK8.10b": return OrthogonalizedLieMarkovRateMatrix.getM8r10bMK();
		case "MK8.16" : return OrthogonalizedLieMarkovRateMatrix.getM8r16MK();
		case "MK8.17" : return OrthogonalizedLieMarkovRateMatrix.getM8r17MK();
		case "MK8.18" : return OrthogonalizedLieMarkovRateMatrix.getM8r18MK();
		case "MK9.20a": return OrthogonalizedLieMarkovRateMatrix.getM9r20aMK();
		case "MK10.12": return OrthogonalizedLieMarkovRateMatrix.getM10r12MK();
		case "MK10.34": return OrthogonalizedLieMarkovRateMatrix.getM10r34MK();
		default : throw new IllegalArgumentException(model);
		}
	}
}
