package lieAlgebra;

import pal.substmodel.GeneralREVRateMatrix;
/*
 * 
 * Licensed under the Creative Commons Zero (CC0) license (https://creativecommons.org/publicdomain/zero/1.0/)
 * 
 */

/**
 * Just a wrapper around Pal's GeneralREVRateMatrix, but with a non-zero lower bound on
 * parameters to ensure it plays nicely with ConjugateDirectionSearch.
 * @author woodhams
 *
 */

/*
 * Normally this file lives in 'palExtensions' project, but I have copied it here.
 * I can't use 'palExtensions' version, because palExtensions uses the unmodified pal-1.5.1 code
 * as its base, but LieAlgebra project uses my highly modified version, for non-time-reversibility support.
 *  
 */

public class ModifiedGREVRateMatrix extends GeneralREVRateMatrix {
	private static final long serialVersionUID = -1665232672437209655L;
	// The entire point of this class:
	public double getRateParameterLowerBound(int parameter) { return 0.00001; }
	
	public ModifiedGREVRateMatrix(int dimension) {
		super(dimension);
	}
	 public ModifiedGREVRateMatrix(int dimension, int[] constraints, double[] specifiedDefaultParameters, int fixedConstraintValue) {
		 super(dimension,constraints,specifiedDefaultParameters,fixedConstraintValue);
	 }

	// F81, or JC if equal base frequencies
	public static final ModifiedGREVRateMatrix modCreateF81() {
		return new ModifiedGREVRateMatrix(4,new int[] {0, 0, 0, 0, 0, 0},new double[] {},0);
	}
	// HKY, or K80/K2ST if equal base frequencies
	public static final ModifiedGREVRateMatrix modCreateHKY() {
		return new ModifiedGREVRateMatrix(4,new int[] {0, 1, 0, 0, 1, 0},new double[] {1},0);
	}
	// TrN (Tamura Nei)
	public static final ModifiedGREVRateMatrix modCreateTrN() {
		return new ModifiedGREVRateMatrix(4,new int[] {0, 1, 0, 0, 2, 0},new double[] {1,1},0);
	}
	// K81/K3ST
	public static final ModifiedGREVRateMatrix modCreateK81() {
		return new ModifiedGREVRateMatrix(4,new int[] {0, 1, 2, 2, 1, 0},new double[] {1,1},0);
	}
	// TVM (Transversional model, named by ModelTest?)
	public static final ModifiedGREVRateMatrix modCreateTVM() {
		return new ModifiedGREVRateMatrix(4,new int[] {0, 1, 2, 3, 1, 4},new double[] {1,1,1,1},0);
	}
	// TIM (Transitional model, named by ModelTest?)
	public static final ModifiedGREVRateMatrix modCreateTIM() {
		return new ModifiedGREVRateMatrix(4,new int[] {0, 1, 2, 2, 3, 0},new double[] {1,1,1},0);
	}
	// GTR, or SYM if equal base frequencies
	public static final ModifiedGREVRateMatrix modCreateGTR() { return new ModifiedGREVRateMatrix(4); }
}
