package lieAlgebra;

import pal.datatype.Nucleotides;
import pal.substmodel.GammaPlusInvRates;
import pal.substmodel.GammaRates;
import pal.substmodel.GeneralRateDistributionSubstitutionModel;
import pal.substmodel.InvariableSites;
import pal.substmodel.NeoRateMatrix;
import pal.substmodel.SingleClassSubstitutionModel;
/*
 * 
 * Licensed under the Creative Commons Zero (CC0) license (https://creativecommons.org/publicdomain/zero/1.0/)
 * 
 */

// quick hack of a class, public members rather than 'get' methods.

// TODO: I create scModel, iModel, gModel every time, even when probably only one will be used.
// (Just because it was easy to program.) May want to reconsider - possibly use a 'get' method
// which creates the model when it is first needed.
public class TestModel {
	public final String name;
	public final NeoRateMatrix q; 
	public final SingleClassSubstitutionModel scModel;
	public final GeneralRateDistributionSubstitutionModel iModel;
	public final GeneralRateDistributionSubstitutionModel gModel;
	public final GeneralRateDistributionSubstitutionModel giModel;
	public final boolean varFreq;
	
	private static final double DEFAULT_INV_SITE_FRAC = 0.1;
	private static final double DEFAULT_GAMMA_PARAM = 1.0;
	private static final int DEFAULT_N_GAMMA_RATE_CLASSES = 4;
	
	public TestModel(String name, NeoRateMatrix q, boolean varFreq, boolean stochasticParams) {

		this.name    = name;
		this.q       = q;
		this.varFreq = varFreq;
		double[] freq = (stochasticParams) ? null : new double[]{0.25,0.25,0.25,0.25};
		if (!stochasticParams) q.setUpperBounds(Configuration.PARAM_UPPER_BOUND);
		this.scModel   = new SingleClassSubstitutionModel
			(
				q,
				Nucleotides.DEFAULT_INSTANCE,
				freq,
				!varFreq
			);
		InvariableSites invSite = new InvariableSites(DEFAULT_INV_SITE_FRAC);
		invSite.setUpperLimit(0.8); // setting this too high causes very many problems for ConjugateDirectionSearch.
		this.iModel = new GeneralRateDistributionSubstitutionModel
			(
				q,
				invSite,
				Nucleotides.DEFAULT_INSTANCE,
				freq,
				!varFreq
			);
		this.gModel = new GeneralRateDistributionSubstitutionModel
		(
			q,
			new GammaRates(DEFAULT_N_GAMMA_RATE_CLASSES, DEFAULT_GAMMA_PARAM),
			Nucleotides.DEFAULT_INSTANCE,
			freq,
			!varFreq
		);
		this.giModel = new GeneralRateDistributionSubstitutionModel
		(
			q,
			new GammaPlusInvRates(DEFAULT_N_GAMMA_RATE_CLASSES, DEFAULT_GAMMA_PARAM, DEFAULT_INV_SITE_FRAC),
			Nucleotides.DEFAULT_INSTANCE,
			freq,
			!varFreq
		);
	}

}
